@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.838';
+our $VERSION = '1.866';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -24,39 +24,34 @@ my ($_has_threads, $_freeze, $_thaw, $_tid, $_oid);
 BEGIN {
    local $@;
 
-   if ($^O eq 'MSWin32' && !$INC{'threads.pm'}) {
-      eval 'use threads; use threads::shared';
+   if ( $^O eq 'MSWin32' && ! $INC{'threads.pm'} ) {
+      eval 'use threads; use threads::shared;';
    }
-   elsif ($INC{'threads.pm'} && !$INC{'threads/shared.pm'}) {
-      eval 'use threads::shared';
+   elsif ( $INC{'threads.pm'} && ! $INC{'threads/shared.pm'} ) {
+      eval 'use threads::shared;';
    }
 
    $_has_threads = $INC{'threads.pm'} ? 1 : 0;
    $_tid = $_has_threads ? threads->tid() : 0;
    $_oid = "$$.$_tid";
 
-   if ($] ge '5.008008' && !$INC{'PDL.pm'}) {
-      eval '
-         use Sereal::Encoder 3.015 qw( encode_sereal );
-         use Sereal::Decoder 3.015 qw( decode_sereal );
-      ';
-      if ( !$@ ) {
+   if ( $] ge '5.008008' && ! $INC{'PDL.pm'} ) {
+      eval 'use Sereal::Encoder 3.015; use Sereal::Decoder 3.015;';
+      if ( ! $@ ) {
          my $_encoder_ver = int( Sereal::Encoder->VERSION() );
          my $_decoder_ver = int( Sereal::Decoder->VERSION() );
          if ( $_encoder_ver - $_decoder_ver == 0 ) {
-            $_freeze = \&encode_sereal,
-            $_thaw   = \&decode_sereal;
+            $_freeze = \&Sereal::Encoder::encode_sereal;
+            $_thaw   = \&Sereal::Decoder::decode_sereal;
          }
       }
    }
 
-   if (!defined $_freeze) {
+   if ( ! defined $_freeze ) {
       require Storable;
-      $_freeze = \&Storable::freeze,
+      $_freeze = \&Storable::freeze;
       $_thaw   = \&Storable::thaw;
    }
-
-   return;
 }
 
 use IO::Handle ();
@@ -88,11 +83,11 @@ BEGIN {
    ## _chunk_id _pids _run_mode _single_dim _thrs _tids _task_wid _wid _wuf
    ## _exiting _exit_pid _last_sref _total_exited _total_running _total_workers
    ## _send_cnt _sess_dir _spawned _state _status _task _task_id _wrk_status
-   ## _init_pid _init_total_workers
+   ## _init_pid _init_total_workers _pids_t _pids_w _pids_c _relayed
    ##
-   ## _bsb_r_sock _bsb_w_sock _bse_r_sock _bse_w_sock _com_r_sock _com_w_sock
-   ## _dat_r_sock _dat_w_sock _que_r_sock _que_w_sock _rla_r_sock _rla_w_sock
-   ## _data_channels _lock_chn _mutex_n
+   ## _bsb_r_sock _bsb_w_sock _com_r_sock _com_w_sock _dat_r_sock _dat_w_sock
+   ## _que_r_sock _que_w_sock _rla_r_sock _rla_w_sock _data_channels
+   ## _lock_chn   _mutex_n
 
    %_valid_fields_new = map { $_ => 1 } qw(
       max_workers tmp_dir use_threads user_tasks task_end task_name freeze thaw
@@ -212,7 +207,7 @@ sub import {
 use constant {
 
    # Max data channels. This cannot be greater than 8 on MSWin32.
-   DATA_CHANNELS  => ($^O eq 'MSWin32') ? 8 : 12,
+   DATA_CHANNELS  => ($^O eq 'MSWin32') ? 8 : 10,
 
    # Max GC size. Undef variable when exceeding size.
    MAX_GC_SIZE    => 1024 * 1024 * 64,
@@ -240,7 +235,9 @@ use constant {
    OUTPUT_B_SYN   => 'B~SYN',  # Barrier sync - begin
    OUTPUT_E_SYN   => 'E~SYN',  # Barrier sync - end
    OUTPUT_S_IPC   => 'S~IPC',  # Change to win32 IPC
+   OUTPUT_C_NFY   => 'C~NFY',  # Chunk ID notification
    OUTPUT_P_NFY   => 'P~NFY',  # Progress notification
+   OUTPUT_R_NFY   => 'R~NFY',  # Relay notification
    OUTPUT_S_DIR   => 'S~DIR',  # Make/get sess_dir
    OUTPUT_T_DIR   => 'T~DIR',  # Make/get tmp_dir
    OUTPUT_I_DLY   => 'I~DLY',  # Interval delay
@@ -266,7 +263,7 @@ use constant {
 my $_mce_count = 0;
 
 sub CLONE {
-   $_tid = threads->tid() if $_has_threads;
+   $_tid = threads->tid() if $INC{'threads.pm'};
 }
 
 sub DESTROY {
@@ -282,7 +279,9 @@ sub DESTROY {
 
 END {
    return unless ( defined $MCE );
-   $MCE->exit if ( exists $MCE->{_wuf} && $MCE->{_pid} eq "$$" );
+
+   my $_pid = $MCE->{_is_thread} ? $$ .'.'. threads->tid() : $$;
+   $MCE->exit if ( exists $MCE->{_wuf} && $MCE->{_pid} eq $_pid );
 
    _end();
 }
@@ -401,7 +400,7 @@ sub new {
    $self{_caller} = $_pkg, $self{_init_pid} = "$$.$_tid";
 
    if (defined $self{use_threads}) {
-      if (!$_has_threads && $self{use_threads} ne '0') {
+      if (!$_has_threads && $self{use_threads}) {
          my $_msg  = "\n";
             $_msg .= "## Please include threads support prior to loading MCE\n";
             $_msg .= "## when specifying use_threads => $self{use_threads}\n";
@@ -416,12 +415,11 @@ sub new {
 
    if (!exists $self{posix_exit}) {
       $self{posix_exit} = 1 if (
-         $^S || ( $_has_threads && $_tid ) || $INC{'Mojo/IOLoop.pm'} ||
+         $^S || $_tid || $INC{'Mojo/IOLoop.pm'} ||
+         $INC{'Coro.pm'} || $INC{'LWP/UserAgent.pm'} || $INC{'stfl.pm'} ||
          $INC{'Curses.pm'} || $INC{'CGI.pm'} || $INC{'FCGI.pm'} ||
-         $INC{'Prima.pm'} || $INC{'Tk.pm'} || $INC{'Wx.pm'} ||
-         $INC{'Gearman/Util.pm'} || $INC{'Gearman/XS.pm'} ||
-         $INC{'Coro.pm'} || $INC{'LWP/UserAgent.pm'} ||
-         $INC{'Win32/GUI.pm'} || $INC{'stfl.pm'}
+         $INC{'Tk.pm'} || $INC{'Wx.pm'} || $INC{'Win32/GUI.pm'} ||
+         $INC{'Gearman/Util.pm'} || $INC{'Gearman/XS.pm'}
       );
    }
 
@@ -448,6 +446,8 @@ sub new {
             _croak("MCE::new: ($_p) is not a valid task constructor argument")
                unless (exists $_valid_fields_task{$_p});
          }
+         $_task->{max_workers} = 0 unless scalar(keys %{ $_task });
+
          $_task->{max_workers} = $self{max_workers}
             unless (defined $_task->{max_workers});
          $_task->{use_threads} = $self{use_threads}
@@ -477,11 +477,14 @@ sub new {
    $self{_last_sref}  = (ref $self{input_data} eq 'SCALAR')
       ? refaddr($self{input_data}) : 0;
 
-   my $_data_channels = ("$$.$_tid" eq $_oid) ? DATA_CHANNELS : 2;
+   my $_data_channels = ("$$.$_tid" eq $_oid)
+      ? ( $INC{'MCE/Channel.pm'} ? 6 : DATA_CHANNELS )
+      : 2;
+
    my $_total_workers = 0;
 
    if (defined $self{user_tasks}) {
-      $_total_workers += $_->{max_workers} for (@{ $self{user_tasks} });
+      $_total_workers += $_->{max_workers} for @{ $self{user_tasks} };
    } else {
       $_total_workers = $self{max_workers};
    }
@@ -492,7 +495,7 @@ sub new {
       ? $_total_workers : $_data_channels;
 
    $self{_lock_chn} = ($_total_workers > $_data_channels) ? 1 : 0;
-   $self{_lock_chn} = 1 if $INC{'MCE/Hobo.pm'};
+   $self{_lock_chn} = 1 if $INC{'MCE/Child.pm'} || $INC{'MCE/Hobo.pm'};
 
    $MCE = \%self if ($MCE->{_wid} == 0);
 
@@ -516,8 +519,11 @@ sub spawn {
    ## Return if workers have already been spawned or if module instance.
    return $self if ($self->{_spawned} || exists $self->{_module_instance});
 
-   lock $_MCE_LOCK if $_has_threads;  # Obtain locks
-   lock $_WIN_LOCK if $_is_MSWin32;
+   lock $_WIN_LOCK if $_is_MSWin32;    # Obtain locks
+   lock $_MCE_LOCK if $_has_threads && $_is_winenv;
+
+   $MCE::_GMUTEX->lock() if ($_tid && $MCE::_GMUTEX);
+   sleep 0.015 if $_tid;
 
    if ($INC{'PDL.pm'}) { local $@;
       eval 'use PDL::IO::Storable' unless $INC{'PDL/IO/Storable.pm'};
@@ -527,7 +533,7 @@ sub spawn {
       local $@; eval 'require Net::HTTP; require Net::HTTPS';
    }
 
-   ## Start the shared-manager process if present.
+   ## Start the shared-manager process if not running.
    MCE::Shared->start() if $INC{'MCE/Shared.pm'};
 
    ## Load input module.
@@ -551,10 +557,6 @@ sub spawn {
       }
    }
 
-   if ("$$.$_tid" ne $_oid && (!$self->{use_threads} || $_is_MSWin32)) {
-      sleep 0.015;
-   }
-
    my $_die_handler  = $SIG{__DIE__};
    my $_warn_handler = $SIG{__WARN__};
 
@@ -579,6 +581,7 @@ sub spawn {
          my $_DAT_W_SOCK = $TOP_HDLR->{_dat_w_sock}->[0];
          print {$_DAT_W_SOCK} OUTPUT_S_IPC.$LF . '0'.$LF;
 
+         MCE::Util::_sock_ready($_DAT_W_SOCK, -1);
          MCE::Util::_sysread($_DAT_W_SOCK, my($_buf), 1);
       }
    }
@@ -599,7 +602,6 @@ sub spawn {
 
    ## Create sockets for IPC.
    MCE::Util::_sock_pair($self, qw(_bsb_r_sock _bsb_w_sock));       # sync
-   MCE::Util::_sock_pair($self, qw(_bse_r_sock _bse_w_sock));       # sync
    MCE::Util::_sock_pair($self, qw(_com_r_sock _com_w_sock));       # core
    MCE::Util::_sock_pair($self, qw(_dat_r_sock _dat_w_sock), $_)    # core
       for (0 .. $_data_channels);
@@ -627,9 +629,11 @@ sub spawn {
    $self->{_pids}   = [], $self->{_thrs}  = [], $self->{_tids} = [];
    $self->{_status} = [], $self->{_state} = [], $self->{_task} = [];
 
-   local $SIG{TTIN}  unless $_is_MSWin32;
-   local $SIG{TTOU}  unless $_is_MSWin32;
-   local $SIG{WINCH} unless $_is_MSWin32;
+   if ($self->{loop_timeout} && !$_is_MSWin32) {
+      $self->{_pids_t} = {}, $self->{_pids_w} = {};
+   }
+
+   local $SIG{TTIN}, local $SIG{TTOU}, local $SIG{WINCH} unless $_is_MSWin32;
 
    if (!defined $self->{user_tasks}) {
       $self->{_total_workers} = $_max_workers;
@@ -643,7 +647,6 @@ sub spawn {
       $self->{_task}->[0] = { _total_workers => $_max_workers };
 
       for my $_i (1 .. $_max_workers) {
-         keys(%{ $self->{_state}->[$_i] }) = 5;
          $self->{_state}->[$_i] = {
             _task => undef, _task_id => undef, _task_wid => undef,
             _params => undef, _chn => $_i % $_data_channels + 1
@@ -696,7 +699,7 @@ sub spawn {
             _total_running => 0, _total_workers => $_task->{max_workers}
          };
          for my $_i (1 .. $_task->{max_workers}) {
-            keys(%{ $self->{_state}->[++$_wid] }) = 5;
+            $_wid += 1;
             $self->{_state}->[$_wid] = {
                _task => $_task, _task_id => $_task_id, _task_wid => $_i,
                _params => undef, _chn => $_wid % $_data_channels + 1
@@ -716,6 +719,8 @@ sub spawn {
 
    $MCE = $self if ($MCE->{_wid} == 0);
 
+   $MCE::_GMUTEX->unlock() if ($_tid && $MCE::_GMUTEX);
+
    return $self;
 }
 
@@ -883,6 +888,14 @@ sub restart_worker {
       _dispatch_child($self, $_wid, $_task, $_task_id, $_task_wid, $_params);
    }
 
+   delete $self->{_retry_cnt};
+
+   if (defined $self->{spawn_delay} && $self->{spawn_delay} > 0.0) {
+      sleep $self->{spawn_delay};
+   } elsif ($_tid || $_is_MSWin32) {
+      sleep 0.045;
+   }
+
    return;
 }
 
@@ -1123,9 +1136,7 @@ sub run {
 
       ## Insert the first message into the queue if defined.
       if (defined $_first_msg) {
-         MCE::Util::_syswrite(
-            $self->{_que_w_sock}, pack($_que_template, 0, $_first_msg)
-         );
+         syswrite($self->{_que_w_sock}, pack($_que_template, 0, $_first_msg));
       }
 
       ## Submit params data to workers.
@@ -1140,9 +1151,8 @@ sub run {
             $self->{_state}[$_wid]{_params} = \%_params_nodata;
          }
 
-         if (defined $_submit_delay && $_submit_delay > 0.0) {
-            sleep $_submit_delay;
-         }
+         sleep $_submit_delay
+            if defined($_submit_delay) && $_submit_delay > 0.0;
       }
    }
 
@@ -1162,11 +1172,11 @@ sub run {
          ## Notify workers to commence processing.
          if ($_is_MSWin32) {
             my $_buf = _sprintf("%${_total_workers}s", "");
-            MCE::Util::_syswrite($self->{_bse_w_sock}, $_buf);
+            syswrite($self->{_bsb_r_sock}, $_buf);
          } else {
-            my $_BSE_W_SOCK = $self->{_bse_w_sock};
+            my $_BSB_R_SOCK = $self->{_bsb_r_sock};
             for my $_i (1 .. $_total_workers) {
-               MCE::Util::_syswrite($_BSE_W_SOCK, $LF);
+               syswrite($_BSB_R_SOCK, $LF);
             }
          }
       }
@@ -1192,9 +1202,9 @@ sub run {
    }
    elsif ($^S || $ENV{'PERL_IPERL_RUNNING'}) {
       if (
+         !$INC{'Mojo/IOLoop.pm'} && !$INC{'Win32/GUI.pm'} &&
          !$INC{'Gearman/XS.pm'} && !$INC{'Gearman/Util.pm'} &&
-         !$INC{'Prima.pm'} && !$INC{'Tk.pm'} && !$INC{'Wx.pm'} &&
-         !$INC{'Mojo/IOLoop.pm'} && !$INC{'Win32/GUI.pm'}
+         !$INC{'Tk.pm'} && !$INC{'Wx.pm'}
       ) {
          # running inside eval or IPerl, check stack trace
          my $_t = Carp::longmess(); $_t =~ s/\teval [^\n]+\n$//;
@@ -1266,12 +1276,11 @@ sub send {
    print({$_COM_R_SOCK} '_data'.$LF), <$_COM_R_SOCK>;
    print({$_COM_R_SOCK} $_len.$LF, $_frozen_data), <$_COM_R_SOCK>;
 
-   if (defined $_submit_delay && $_submit_delay > 0.0) {
-      sleep $_submit_delay;
-   }
-
    $self->{_send_cnt} += 1;
 
+   sleep $_submit_delay
+      if defined($_submit_delay) && $_submit_delay > 0.0;
+
    return $self;
 }
 
@@ -1291,7 +1300,7 @@ sub shutdown {
    return unless $self->{_spawned};
 
    ## Return if signaled.
-   if (defined $MCE::Signal::KILLED) {
+   if ($MCE::Signal::KILLED) {
       if (defined $self->{_sess_dir}) {
          my $_sess_dir = delete $self->{_sess_dir};
          rmdir $_sess_dir if -d $_sess_dir;
@@ -1345,9 +1354,8 @@ sub shutdown {
    $_COM_R_SOCK = undef;
 
    MCE::Util::_destroy_socks($self, qw(
-      _bsb_w_sock _bsb_r_sock _bse_w_sock _bse_r_sock
-      _com_w_sock _com_r_sock _dat_w_sock _dat_r_sock
-      _rla_w_sock _rla_r_sock
+      _bsb_w_sock _bsb_r_sock _com_w_sock _com_r_sock
+      _dat_w_sock _dat_r_sock _rla_w_sock _rla_r_sock
    ));
 
    ($_is_MSWin32)
@@ -1371,6 +1379,11 @@ sub shutdown {
    $self->{_total_workers} = 0;
    $self->{_sess_dir} = undef;
 
+   if ($self->{loop_timeout}) {
+      delete $self->{_pids_t};
+      delete $self->{_pids_w};
+   }
+
    return;
 }
 
@@ -1393,7 +1406,7 @@ sub sync {
    my $_chn        = $self->{_chn};
    my $_DAT_W_SOCK = $self->{_dat_w_sock}->[0];
    my $_BSB_R_SOCK = $self->{_bsb_r_sock};
-   my $_BSE_R_SOCK = $self->{_bse_r_sock};
+   my $_BSB_W_SOCK = $self->{_bsb_w_sock};
    my $_buf;
 
    local $\ = undef if (defined $\);
@@ -1409,7 +1422,8 @@ sub sync {
    print {$_DAT_W_SOCK} OUTPUT_E_SYN.$LF . $_chn.$LF;
 
    ## Wait until all workers from (task_id 0) have un-synced.
-   MCE::Util::_sysread($_BSE_R_SOCK, $_buf, 1);
+   MCE::Util::_sock_ready($_BSB_W_SOCK, -1) if $_is_MSWin32;
+   MCE::Util::_sysread($_BSB_W_SOCK, $_buf, 1);
 
    return;
 }
@@ -1435,9 +1449,7 @@ sub yield {
    chomp($_delay = <$_DAU_W_SOCK>);
    $_DAT_LOCK->unlock() if $_lock_chn;
 
-   sleep $_delay if ($_delay > 0.0);
-
-   return;
+   MCE::Util::_sleep( $_delay );
 }
 
 ###############################################################################
@@ -1460,7 +1472,7 @@ sub abort {
 
       if ($_abort_msg > 0) {
          MCE::Util::_sysread($_QUE_R_SOCK, my($_next), $_que_read_size);
-         MCE::Util::_syswrite($_QUE_W_SOCK, pack($_que_template, 0, $_abort_msg));
+         syswrite($_QUE_W_SOCK, pack($_que_template, 0, $_abort_msg));
       }
 
       if ($self->{_wid} > 0) {
@@ -1486,7 +1498,7 @@ sub exit {
 
    my $_exit_status = (defined $_[0]) ? $_[0] : $?;
    my $_exit_msg    = (defined $_[1]) ? $_[1] : '';
-   my $_exit_id     = (defined $_[2]) ? $_[2] : '';
+   my $_exit_id     = (defined $_[2]) ? $_[2] : $self->chunk_id;
 
    @_ = ();
 
@@ -1500,10 +1512,26 @@ sub exit {
    my $_lock_chn   = $self->{_lock_chn};
    my $_task_id    = $self->{_task_id};
 
-   unless ($self->{_exiting}) {
+   unless ( $self->{_exiting} ) {
       $self->{_exiting} = 1;
 
-      ## Check nested Hobo workers not yet joined.
+      my $_pid = $self->{_is_thread} ? $$ .'.'. threads->tid() : $$;
+      my $_max_retries = $self->{max_retries};
+      my $_chunk_id = $self->{_chunk_id};
+
+      if ( defined $self->{init_relay} && !$self->{_relayed} && !$_task_id &&
+           exists $self->{_wuf} && $self->{_pid} eq $_pid ) {
+
+         $self->{_retry_cnt} = -1 unless defined( $self->{_retry_cnt} );
+
+         if ( !$_max_retries || ++$self->{_retry_cnt} == $_max_retries ) {
+            MCE::relay { warn "Error: chunk $_chunk_id failed\n" if $_chunk_id };
+         }
+      }
+
+      ## Check for nested workers not yet joined.
+      MCE::Child->finish('MCE') if $INC{'MCE/Child.pm'};
+
       MCE::Hobo->finish('MCE')
          if ( $INC{'MCE/Hobo.pm'} && MCE::Hobo->can('_clear') );
 
@@ -1514,7 +1542,7 @@ sub exit {
       $_DAT_LOCK->lock() if $_lock_chn;
 
       if ($self->{_retry} && $self->{_retry}->[2]--) {
-         my $_buf = $self->{freeze}($self->{_retry});
+         $_exit_status = 0; my $_buf = $self->{freeze}($self->{_retry});
          print({$_DAT_W_SOCK} OUTPUT_W_EXT.$LF . $_chn.$LF),
          print({$_DAU_W_SOCK}
             $_task_id.$LF . $self->{_wid}.$LF . $self->{_exit_pid}.$LF .
@@ -1608,17 +1636,20 @@ sub do {
    my $self = shift; $self = $MCE unless ref($self);
    my $_pkg = caller() eq 'MCE' ? caller(1) : caller();
 
-   _croak('MCE::do: method is not allowed by the manager process')
-      unless ($self->{_wid});
    _croak('MCE::do: (code ref) is not supported')
       if (ref $_[0] eq 'CODE');
-
    _croak('MCE::do: (callback) is not specified')
       unless (defined ( my $_func = shift ));
 
    $_func = $_pkg.'::'.$_func if (index($_func, ':') < 0);
 
-   return _do_callback($self, $_func, [ @_ ]);
+   if ($self->{_wid}) {
+      return _do_callback($self, $_func, [ @_ ]);
+   }
+   else {
+      no strict 'refs';
+      return $_func->(@_);
+   }
 }
 
 ## Gather method.
@@ -1659,20 +1690,27 @@ sub gather {
       $_dest = (exists $_sendto_lkup{$_to}) ? $_sendto_lkup{$_to} : undef;
 
       if (!defined $_dest) {
-         if (ref $_to && defined (my $_fd = fileno($_to))) {
+         my $_fd;
+
+         if (ref $_to && ( defined ($_fd = fileno($_to)) ||
+                           defined ($_fd = $_to->fileno) )) {
+
             if (my $_ob = tied *{ $_to }) {
                if (ref $_ob eq 'IO::TieCombine::Handle') {
                   $_fd = 1 if (lc($_ob->{slot_name}) eq 'stdout');
                   $_fd = 2 if (lc($_ob->{slot_name}) eq 'stderr');
                }
             }
+
             my $_data_ref = (scalar @_ == 1) ? \$_[0] : \join('', @_);
             return _do_send_glob($self, $_to, $_fd, $_data_ref);
          }
+
          if (defined $_to && $_to =~ /$_v2_regx/o) {
             $_dest  = (exists $_sendto_lkup{$1}) ? $_sendto_lkup{$1} : undef;
             $_value = $2;
          }
+
          if (!defined $_dest || ( !defined $_value && (
                $_dest == SENDTO_FILEV2 || $_dest == SENDTO_FD
          ))) {
@@ -1710,13 +1748,16 @@ sub print {
    my $self = shift; $self = $MCE unless ref($self);
    my ($_fd, $_glob, $_data_ref);
 
-   if (ref $_[0] && defined ($_fd = fileno($_[0]))) {
+   if (ref $_[0] && ( defined ($_fd = fileno($_[0])) ||
+                      defined ($_fd = $_[0]->fileno) )) {
+
       if (my $_ob = tied *{ $_[0] }) {
          if (ref $_ob eq 'IO::TieCombine::Handle') {
             $_fd = 1 if (lc($_ob->{slot_name}) eq 'stdout');
             $_fd = 2 if (lc($_ob->{slot_name}) eq 'stderr');
          }
       }
+
       $_glob = shift;
    }
 
@@ -1737,13 +1778,16 @@ sub printf {
    my $self = shift; $self = $MCE unless ref($self);
    my ($_fd, $_glob, $_fmt, $_data);
 
-   if (ref $_[0] && defined ($_fd = fileno($_[0]))) {
+   if (ref $_[0] && ( defined ($_fd = fileno($_[0])) ||
+                      defined ($_fd = $_[0]->fileno) )) {
+
       if (my $_ob = tied *{ $_[0] }) {
          if (ref $_ob eq 'IO::TieCombine::Handle') {
             $_fd = 1 if (lc($_ob->{slot_name}) eq 'stdout');
             $_fd = 2 if (lc($_ob->{slot_name}) eq 'stderr');
          }
       }
+
       $_glob = shift;
    }
 
@@ -1759,13 +1803,16 @@ sub say {
    my $self = shift; $self = $MCE unless ref($self);
    my ($_fd, $_glob, $_data);
 
-   if (ref $_[0] && defined ($_fd = fileno($_[0]))) {
+   if (ref $_[0] && ( defined ($_fd = fileno($_[0])) ||
+                      defined ($_fd = $_[0]->fileno) )) {
+
       if (my $_ob = tied *{ $_[0] }) {
          if (ref $_ob eq 'IO::TieCombine::Handle') {
             $_fd = 1 if (lc($_ob->{slot_name}) eq 'stdout');
             $_fd = 2 if (lc($_ob->{slot_name}) eq 'stderr');
          }
       }
+
       $_glob = shift;
    }
 
@@ -1788,15 +1835,18 @@ sub _exit {
    delete $self->{_wuf}; _end();
 
    ## Exit thread/child process.
-   $SIG{__DIE__}  = sub { } unless $_tid;
-   $SIG{__WARN__} = sub { };
+   $SIG{__DIE__}  = sub {} unless $_tid;
+   $SIG{__WARN__} = sub {};
 
    threads->exit(0) if $self->{use_threads};
 
    if (! $_tid) {
       $SIG{HUP} = $SIG{INT} = $SIG{QUIT} = $SIG{TERM} = sub {
-         $SIG{$_[0]} = $SIG{INT} = sub { };
-         CORE::kill($_[0], getppid()) if ($_[0] eq 'INT' && !$_is_MSWin32);
+         $SIG{$_[0]} = $SIG{INT} = $SIG{TERM} = sub {};
+
+         CORE::kill($_[0], getppid())
+            if (($_[0] eq 'INT' || $_[0] eq 'TERM') && $^O ne 'MSWin32');
+
          CORE::kill('KILL', $$);
       };
    }
@@ -1831,7 +1881,7 @@ sub _make_sessdir {
       $self->{tmp_dir} = MCE::Signal::_make_tmpdir()
          unless defined $self->{tmp_dir};
 
-      my $_mce_tid = $_has_threads ? threads->tid() : '';
+      my $_mce_tid = $INC{'threads.pm'} ? threads->tid() : '';
          $_mce_tid = '' unless defined $self->{_mce_tid};
 
       my $_mce_sid = $$ .'.'. $_mce_tid .'.'. (++$_mce_count);
@@ -1855,7 +1905,6 @@ sub _make_sessdir {
 
 sub _sprintf {
    my ($_fmt, $_arg) = @_;
-
    # remove tainted'ness
    ($_fmt) = $_fmt =~ /(.*)/;
 
@@ -1923,11 +1972,16 @@ sub _dispatch {
    ## To avoid (Scalars leaked: N) messages; fixed in Perl 5.12.x
    @_ = ();
 
-   $ENV{'PERL_MCE_IPC'} = 'win32' if ($_is_MSWin32 && $INC{'MCE/Hobo.pm'});
+   $ENV{'PERL_MCE_IPC'} = 'win32' if ( $_is_MSWin32 && (
+      defined($self->{max_retries}) ||
+      $INC{'MCE/Child.pm'} ||
+      $INC{'MCE/Hobo.pm'}
+   ));
 
-   if (!$_is_thread && UNIVERSAL::can('Prima', 'cleanup')) {
-      no warnings 'redefine'; local $@; eval '*Prima::cleanup = sub {}';
-   }
+   delete $self->{_relayed};
+
+   $self->{_is_thread} = $_is_thread;
+   $self->{_pid}       = $_is_thread ? $$ .'.'. threads->tid() : $$;
 
    ## Sets the seed of the base generator uniquely between workers.
    ## The new seed is computed using the current seed and $_wid value.
@@ -1935,15 +1989,19 @@ sub _dispatch {
    ## results (non-thread workers only). Ditto for Math::Prime::Util,
    ## Math::Random, and Math::Random::MT::Auto.
 
-   if (!$self->{use_threads}) {
+   {
       my ($_wid, $_seed) = ($_args[1], $self->{_seed});
       srand(abs($_seed - ($_wid * 100000)) % 2147483560);
 
-      Math::Prime::Util::srand(abs($_seed - ($_wid * 100000)) % 2147483560)
-         if ( $INC{'Math/Prime/Util.pm'} );
+      if (!$self->{use_threads}) {
+         Math::Prime::Util::srand(abs($_seed - ($_wid * 100000)) % 2147483560)
+            if ( $INC{'Math/Prime/Util.pm'} );
 
-      MCE::Hobo->_clear()
-         if ( $INC{'MCE/Hobo.pm'} && MCE::Hobo->can('_clear') );
+         MCE::Hobo->_clear()
+            if ( $INC{'MCE/Hobo.pm'} && MCE::Hobo->can('_clear') );
+
+         MCE::Child->_clear() if $INC{'MCE/Child.pm'};
+      }
    }
 
    if (!$self->{use_threads} && $INC{'Math/Random.pm'}) {
@@ -1969,8 +2027,6 @@ sub _dispatch {
 
    ## Run.
 
-   $self->{_pid} = ($_is_thread) ? $$ .'.'. threads->tid() : $$;
-
    _worker_main(@_args, \@_plugin_worker_init);
 
    _exit($self);
@@ -2000,9 +2056,8 @@ sub _dispatch_thread {
    push @{ $self->{_thrs} }, $_thr;
    push @{ $self->{_tids} }, $_thr->tid();
 
-   if (defined $self->{spawn_delay} && $self->{spawn_delay} > 0.0) {
-      sleep $self->{spawn_delay};
-   }
+   sleep $self->{spawn_delay}
+      if defined($self->{spawn_delay}) && $self->{spawn_delay} > 0.0;
 
    return;
 }
@@ -2029,10 +2084,14 @@ sub _dispatch_child {
 
    push @{ $self->{_pids} }, $_pid;
 
-   if (defined $self->{spawn_delay} && $self->{spawn_delay} > 0.0) {
-      sleep $self->{spawn_delay};
+   if ($self->{loop_timeout} && !$_is_MSWin32) {
+      $self->{_pids_t}{$_pid} = $_task_id;
+      $self->{_pids_w}{$_pid} = $_wid;
    }
 
+   sleep $self->{spawn_delay}
+      if defined($self->{spawn_delay}) && $self->{spawn_delay} > 0.0;
+
    return;
 }
 
