@@ -1,6 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## File_path and Scalar_ref input reader.
+## File path and Scalar reference input reader.
 ##
 ## This package provides the read handle method used internally by the worker
 ## process. Distribution follows a bank-queuing model.
@@ -14,7 +14,7 @@ package MCE::Core::Input::Handle;
 use strict;
 use warnings;
 
-our $VERSION = '1.838';
+our $VERSION = '1.866';
 
 ## Items below are folded into MCE.
 
@@ -38,7 +38,7 @@ sub _systell {
    # To minimize memory consumption, SEEK_CUR equals 1 on most platforms.
    # e.g. use Fcntl qw(SEEK_CUR);
 
-   MCE::Util::_sysseek($_[0], 0, 1);
+   sysseek($_[0], 0, 1);
 }
 
 sub _worker_read_handle {
@@ -67,15 +67,15 @@ sub _worker_read_handle {
       $_pid = $INC{'threads.pm'} ? $$ .'.'. threads->tid() : $$;
 
       # inlined for performance
-      if ($self->{_data_channels} > 6) {
-         $_DAT_LOCK = $self->{'_mutex_'.( $self->{_wid} % 6 + 1 )};
+      if ($self->{_data_channels} > 5) {
+         $_DAT_LOCK = $self->{'_mutex_'.( $self->{_wid} % 5 + 1 )};
       }
       $_dat_ex = sub {
          MCE::Util::_sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
             unless $_DAT_LOCK->{ $_pid };
       };
       $_dat_un = sub {
-         MCE::Util::_syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
+         syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
             if $_DAT_LOCK->{ $_pid };
       };
    }
@@ -117,7 +117,7 @@ sub _worker_read_handle {
       ($_chunk_id, $_offset_pos) = unpack($_que_template, $_next);
 
       if ($_offset_pos >= $_data_size) {
-         MCE::Util::_syswrite($_QUE_W_SOCK, pack($_que_template, 0, $_offset_pos));
+         syswrite($_QUE_W_SOCK, pack($_que_template, 0, $_offset_pos));
          $_dat_un->() if $_lock_chn;
          close $_IN_FILE; undef $_IN_FILE;
          return;
@@ -172,7 +172,7 @@ sub _worker_read_handle {
             }
          }
 
-         MCE::Util::_syswrite(
+         syswrite(
             $_QUE_W_SOCK, pack($_que_template, $_chunk_id, tell $_IN_FILE)
          );
          $_dat_un->() if $_lock_chn;
@@ -181,7 +181,7 @@ sub _worker_read_handle {
          local $/ = $_RS if ($/ ne $_RS);
 
          if ($_parallel_io && $_RS eq $LF) {
-            MCE::Util::_syswrite(
+            syswrite(
                $_QUE_W_SOCK,
                pack($_que_template, $_chunk_id, $_offset_pos + $_chunk_size)
             );
@@ -195,8 +195,8 @@ sub _worker_read_handle {
             }
 
             if ($_proc_type == READ_FILE) {
-               MCE::Util::_sysseek($_IN_FILE, tell( $_IN_FILE ), 0);
-               MCE::Util::_sysread($_IN_FILE, $_, $_tmp_cs, $_p);
+               sysseek($_IN_FILE, tell( $_IN_FILE ), 0);
+               sysread($_IN_FILE, $_, $_tmp_cs, $_p);
                seek $_IN_FILE, _systell($_IN_FILE), 0;
             }
             else {
@@ -207,8 +207,8 @@ sub _worker_read_handle {
          }
          else {
             if ($_proc_type == READ_FILE) {
-               MCE::Util::_sysseek($_IN_FILE, $_offset_pos, 0);
-               MCE::Util::_sysread($_IN_FILE, $_, $_chunk_size, $_p);
+               sysseek($_IN_FILE, $_offset_pos, 0);
+               sysread($_IN_FILE, $_, $_chunk_size, $_p);
                seek $_IN_FILE, _systell($_IN_FILE), 0;
             }
             else {
@@ -218,7 +218,7 @@ sub _worker_read_handle {
 
             $_ .= <$_IN_FILE>;
 
-            MCE::Util::_syswrite(
+            syswrite(
                $_QUE_W_SOCK, pack($_que_template, $_chunk_id, tell $_IN_FILE)
             );
             $_dat_un->() if $_lock_chn;
@@ -268,3 +268,36 @@ sub _worker_read_handle {
 
 1;
 
+__END__
+
+###############################################################################
+## ----------------------------------------------------------------------------
+## Module usage.
+##
+###############################################################################
+
+=head1 NAME
+
+MCE::Core::Input::Handle - File path and Scalar reference input reader
+
+=head1 VERSION
+
+This document describes MCE::Core::Input::Handle version 1.866
+
+=head1 DESCRIPTION
+
+This package provides the read handle method used internally by the worker
+process. Distribution follows a bank-queuing model.
+
+There is no public API.
+
+=head1 SEE ALSO
+
+The syntax for the C<input_data> option is described in L<MCE::Core>.
+
+=head1 AUTHOR
+
+Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
+
+=cut
+
