@@ -14,7 +14,7 @@ package MCE::Core::Input::Sequence;
 use strict;
 use warnings;
 
-our $VERSION = '1.838';
+our $VERSION = '1.866';
 
 ## Items below are folded into MCE.
 
@@ -56,15 +56,15 @@ sub _worker_sequence_queue {
       $_pid = $INC{'threads.pm'} ? $$ .'.'. threads->tid() : $$;
 
       # inlined for performance
-      if ($self->{_data_channels} > 6) {
-         $_DAT_LOCK = $self->{'_mutex_'.( $self->{_wid} % 6 + 1 )};
+      if ($self->{_data_channels} > 5) {
+         $_DAT_LOCK = $self->{'_mutex_'.( $self->{_wid} % 5 + 1 )};
       }
       $_dat_ex = sub {
          MCE::Util::_sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
             unless $_DAT_LOCK->{ $_pid };
       };
       $_dat_un = sub {
-         MCE::Util::_syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
+         syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
             if $_DAT_LOCK->{ $_pid };
       };
    }
@@ -102,12 +102,12 @@ sub _worker_sequence_queue {
       ($_chunk_id, $_offset) = unpack($_que_template, $_next);
 
       if ($_offset >= $_abort) {
-         MCE::Util::_syswrite($_QUE_W_SOCK, pack($_que_template, 0, $_offset));
+         syswrite($_QUE_W_SOCK, pack($_que_template, 0, $_offset));
          $_dat_un->() if $_lock_chn;
          return;
       }
 
-      MCE::Util::_syswrite(
+      syswrite(
          $_QUE_W_SOCK, pack($_que_template, $_chunk_id + 1, $_offset + 1)
       );
 
@@ -217,3 +217,36 @@ sub _worker_sequence_queue {
 
 1;
 
+__END__
+
+###############################################################################
+## ----------------------------------------------------------------------------
+## Module usage.
+##
+###############################################################################
+
+=head1 NAME
+
+MCE::Core::Input::Sequence - Sequence of numbers (for task_id == 0)
+
+=head1 VERSION
+
+This document describes MCE::Core::Input::Sequence version 1.866
+
+=head1 DESCRIPTION
+
+This package provides a sequence of numbers used internally by the worker
+process. Distribution follows a bank-queuing model.
+
+There is no public API.
+
+=head1 SEE ALSO
+
+The syntax for the C<sequence> option is described in L<MCE::Core>.
+
+=head1 AUTHOR
+
+Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
+
+=cut
+
