@@ -14,7 +14,7 @@ package MCE::Core::Validation;
 use strict;
 use warnings;
 
-our $VERSION = '1.838';
+our $VERSION = '1.866';
 
 ## Items below are folded into MCE.
 
@@ -56,9 +56,6 @@ sub _validate_args {
    if (defined $_s->{user_tasks}) {
       for my $_t (@{ $_s->{user_tasks} }) {
          _validate_args_s($_s, $_t);
-
-         _croak("$_tag: (task_end) is not a CODE reference")
-            if ($_t->{task_end} && ref $_t->{task_end} ne 'CODE');
       }
    }
 
@@ -105,7 +102,7 @@ sub _validate_args_s {
    _croak("$_tag: (max_retries) is not valid")
       if ($_s->{max_retries} && $_s->{max_retries} !~ /\A\d+\z/);
 
-   for my $_k (qw(progress user_begin user_end user_func)) {
+   for my $_k (qw(progress user_begin user_end user_func task_end)) {
       _croak("$_tag: ($_k) is not a CODE reference")
          if ($_s->{$_k} && ref $_s->{$_k} ne 'CODE');
    }
@@ -195,7 +192,7 @@ sub _validate_args_s {
 
       $_i->{max_nodes} = 1 unless (exists $_i->{max_nodes});
       $_i->{node_id}   = 1 unless (exists $_i->{node_id});
-      $_i->{_time}     = time;
+      $_i->{_time}     = MCE::Util::_time();
    }
 
    return;
@@ -256,7 +253,7 @@ sub _parse_chunk_size {
          # Iterators may optionally use chunk_size to determine how much
          # to return per iteration. The default is 1 for MCE Models, same
          # as for the Core API. The user_func receives an array_ref
-         # regardless if 1 or higher.
+         # regardless if 1 or greater.
          #
          # sub make_iter {
          #    ...
@@ -351,7 +348,7 @@ sub _parse_max_workers {
       $_ncpu_ul = 8 if ($_ncpu_ul > 8);
 
       if ($1 && $2) {
-         local $@; $_max_workers = eval "int($_ncpu_ul $1 $2 + 0.5)";
+         local $@; $_max_workers = eval "int($_ncpu_ul $1 $2 + 0.5)"; ## no critic
          $_max_workers = 1 if (!$_max_workers || $_max_workers < 1);
          $_max_workers = $_ncpu if ($_max_workers > $_ncpu);
       }
@@ -380,3 +377,32 @@ sub _validate_number {
 
 1;
 
+__END__
+
+###############################################################################
+## ----------------------------------------------------------------------------
+## Module usage.
+##
+###############################################################################
+
+=head1 NAME
+
+MCE::Core::Validation - Core validation methods for Many-Core Engine
+
+=head1 VERSION
+
+This document describes MCE::Core::Validation version 1.866
+
+=head1 DESCRIPTION
+
+This package provides validation methods used internally by the manager
+process.
+
+There is no public API.
+
+=head1 AUTHOR
+
+Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
+
+=cut
+
