@@ -14,13 +14,12 @@ package MCE::Core::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '1.838';
+our $VERSION = '1.866';
 
-my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
-my $_tid = $_has_threads ? threads->tid() : 0;
+my $_tid = $INC{'threads.pm'} ? threads->tid() : 0;
 
 sub CLONE {
-   $_tid = threads->tid() if $_has_threads;
+   $_tid = threads->tid() if $INC{'threads.pm'};
 }
 
 ## Items below are folded into MCE.
@@ -111,7 +110,9 @@ use bytes;
 
    sub _do_callback {
 
-      my ($self, $_buf, $_aref);  ($self, $_val, $_aref) = @_;
+      my ($self, $_buf, $_aref);
+
+      ($self, $_val, $_aref) = @_;
 
       unless (defined wantarray) {
          $_wa = WANTS_UNDEF;
@@ -153,7 +154,7 @@ use bytes;
          local $/ = $LF if ($/ ne $LF);
          chomp(my $_len = <$_DAU_W_SOCK>);
 
-         my $_frozen = chop($_len);
+         my $_frozen = chop $_len;
          read $_DAU_W_SOCK, my($_buf), $_len;
          $_dat_un->() if $_lock_chn;
 
@@ -256,6 +257,9 @@ use bytes;
             _do_send($self, SENDTO_FD, $_fd, $_data_ref);
          }
       }
+      elsif ($_glob->can('print')) {
+         $_glob->print(${ $_data_ref });
+      }
       else {
          require Symbol unless $INC{'Symbol.pm'};
          my $_fh = Symbol::qualify_to_ref($_glob, caller);
@@ -282,13 +286,13 @@ use bytes;
       if ($_lock_chn) {
          # inlined for performance
          $_dat_ex = sub {
-            my $_pid = $_has_threads ? $$ .'.'. $_tid : $$;
+            my $_pid = $_tid ? $$ .'.'. $_tid : $$;
             MCE::Util::_sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
                unless $_DAT_LOCK->{ $_pid };
          };
          $_dat_un = sub {
-            my $_pid = $_has_threads ? $$ .'.'. $_tid : $$;
-            MCE::Util::_syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
+            my $_pid = $_tid ? $$ .'.'. $_tid : $$;
+            syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
                if $_DAT_LOCK->{ $_pid };
          };
       }
@@ -320,6 +324,9 @@ use bytes;
       my ($self, $_chunk, $_chunk_id) = @_;
       my $_size = 0;
 
+      delete $self->{_relayed};
+      $self->{_chunk_id} = $_chunk_id;
+
       if ($self->{progress} && $self->{_task_id} == 0) {
          # use_slurpio
          if (ref $_chunk eq 'SCALAR') {
@@ -342,10 +349,22 @@ use bytes;
          }
       }
 
-      $self->{_retry} = [ $_chunk, $_chunk_id, $self->{max_retries} ]
-         if ($self->{max_retries});
+      if ($self->{max_retries}) {
+         $self->{_retry} = [ $_chunk, $_chunk_id, $self->{max_retries} ];
+      }
+
+      if ($self->{loop_timeout} && $self->{_task_id} == 0 &&
+          defined $self->{init_relay} && !$self->{_is_thread} &&
+          $^O ne 'MSWin32') {
+
+         local $\ = undef if (defined $\);
+
+         $_dat_ex->() if $_lock_chn;
+         print({$_DAT_W_SOCK} OUTPUT_C_NFY.$LF . $_chn.$LF),
+         print({$_DAU_W_SOCK} "$$:$_chunk_id".$LF);
+         $_dat_un->() if $_lock_chn;
+      }
 
-      $self->{_chunk_id} = $_chunk_id;
       $_user_func->($self, $_chunk, $_chunk_id);
 
       if ($self->{progress} && $self->{_task_id} == 0) {
@@ -419,7 +438,9 @@ sub _worker_do {
    if (defined $self->{user_begin}) {
       $self->{_chunk_id} = 0;
       $self->{user_begin}($self, $_task_id, $_task_name);
-      $self->sync() if ($_task_id == 0 && defined $self->{init_relay});
+      if ($_task_id == 0 && defined $self->{init_relay} && !$self->{_retry}) {
+         $self->sync();
+      }
    }
 
    ## Retry chunk if previous attempt died.
@@ -471,6 +492,9 @@ sub _worker_do {
       _worker_read_handle($self, READ_MEMORY, $self->{input_data});
    }
    elsif (defined $self->{user_func}) {
+      if ($self->{max_retries}) {
+         $self->{_retry} = [ undef, 0, $self->{max_retries} ];
+      }
       $self->{_chunk_id} = 0;
       $self->{user_func}->($self);
    }
@@ -488,8 +512,9 @@ sub _worker_do {
 
    delete $self->{_wuf};
 
-   ## Check nested Hobo workers not yet joined.
-   MCE::Hobo->finish('MCE') if $INC{'MCE/Hobo.pm'};
+   ## Check for nested workers not yet joined.
+   MCE::Child->finish('MCE') if $INC{'MCE/Child.pm'};
+   MCE::Hobo->finish('MCE')  if $INC{'MCE/Hobo.pm'};
 
    ## Notify the main process a worker has completed.
    local $\ = undef if (defined $\);
@@ -525,54 +550,35 @@ sub _worker_loop {
    my $_COM_LOCK   = $self->{_com_lock};
    my $_COM_W_SOCK = $self->{_com_w_sock};
    my $_job_delay  = $self->{job_delay};
-   my $_wid        = $self->{_wid};
 
-   if ($^O eq 'MSWin32') {
-      lock $MCE::_WIN_LOCK;
-   }
+   if ($^O eq 'MSWin32') { lock $MCE::_WIN_LOCK; }
 
    while (1) {
-
       {
          local $\ = undef; local $/ = $LF;
          $_COM_LOCK->lock();
 
          ## Wait for the next job request.
-         $_response = <$_COM_W_SOCK>;
-         print {$_COM_W_SOCK} $_wid.$LF;
+         $_response = <$_COM_W_SOCK>, print {$_COM_W_SOCK} $self->{_wid}.$LF;
 
          ## Return if instructed to exit.
-         if ($_response eq "_exit\n") {
-            $_COM_LOCK->unlock();
-            return;
-         }
+         $_COM_LOCK->unlock(), return if ($_response eq "_exit\n");
 
          ## Process send request.
          if ($_response eq "_data\n") {
-            chomp($_len = <$_COM_W_SOCK>),
-            read($_COM_W_SOCK, $_buf, $_len);
+            chomp($_len = <$_COM_W_SOCK>), read($_COM_W_SOCK, $_buf, $_len);
+            print({$_COM_W_SOCK} $LF), $_COM_LOCK->unlock();
+            $self->{user_data} = $self->{thaw}($_buf), undef $_buf;
 
-            print {$_COM_W_SOCK} $LF;
-            $_COM_LOCK->unlock();
-
-            $self->{user_data} = $self->{thaw}($_buf);
-            undef $_buf;
-
-            if (defined $_job_delay && $_job_delay > 0.0) {
-               sleep $_job_delay * $_wid;
-            }
+            sleep $_job_delay * $self->{_wid}
+               if defined($_job_delay) && $_job_delay > 0.0;
          }
 
          ## Process normal request.
          elsif ($_response =~ /\d+/) {
-            chomp($_len = <$_COM_W_SOCK>),
-            read($_COM_W_SOCK, $_buf, $_len);
-
-            print {$_COM_W_SOCK} $LF;
-            $_COM_LOCK->unlock();
-
-            $_params_ref = $self->{thaw}($_buf);
-            undef $_buf;
+            chomp($_len = <$_COM_W_SOCK>), read($_COM_W_SOCK, $_buf, $_len);
+            print({$_COM_W_SOCK} $LF), $_COM_LOCK->unlock();
+            $_params_ref = $self->{thaw}($_buf), undef $_buf;
          }
 
          ## Leave loop if invalid response.
@@ -585,12 +591,11 @@ sub _worker_loop {
       _worker_do($self, {}), next if ($_response eq "_data\n");
 
       ## Wait here until MCE completes job submission to all workers.
-      MCE::Util::_sysread($self->{_bse_r_sock}, my($_b), 1);
+      MCE::Util::_sysread($self->{_bsb_w_sock}, my($_b), 1);
 
       ## Normal request.
-      if (defined $_job_delay && $_job_delay > 0.0) {
-         sleep $_job_delay * $_wid;
-      }
+      sleep $_job_delay * $self->{_wid}
+         if defined($_job_delay) && $_job_delay > 0.0;
 
       _worker_do($self, $_params_ref); undef $_params_ref;
    }
@@ -600,7 +605,7 @@ sub _worker_loop {
 
    $_COM_LOCK->unlock();
 
-   die "Worker ($self->{_wid}) has ended prematurely";
+   die "Error: worker $self->{_wid} has ended prematurely";
 }
 
 ###############################################################################
@@ -630,7 +635,7 @@ sub _worker_main {
    my $_use_threads = (defined $_task->{use_threads})
       ? $_task->{use_threads} : $self->{use_threads};
 
-   if ($_has_threads && $_use_threads) {
+   if ($INC{'threads.pm'} && $_use_threads) {
       $self->{_exit_pid} = 'TID_' . $_tid;
    } else {
       $self->{_exit_pid} = 'PID_' . $$;
@@ -638,9 +643,11 @@ sub _worker_main {
 
    my $_running_inside_eval = $^S;
 
+   local $SIG{SEGV} = sub { $self->exit(11) };
+
    local $SIG{__DIE__} = sub {
       if (!defined $^S || $^S) {
-         if ( ($_has_threads && $_tid != 0) ||
+         if ( ($INC{'threads.pm'} && $_tid != 0) ||
                $ENV{'PERL_IPERL_RUNNING'} ||
                $_running_inside_eval
          ) {
@@ -658,9 +665,9 @@ sub _worker_main {
          }
       }
 
-      local $SIG{__DIE__}; local $\ = undef;
+      $SIG{__DIE__} = $SIG{__WARN__} = sub {};
       my $_die_msg = (defined $_[0]) ? $_[0] : '';
-      print {*STDERR} $_die_msg;
+      local $\ = undef; print {*STDERR} $_die_msg;
 
       $self->exit(255, $_die_msg, $self->{_chunk_id});
    };
@@ -699,19 +706,15 @@ sub _worker_main {
 
    ## Call MCE::Shared's init routine if present; enables parallel IPC.
    ## For threads, init is called automatically via the CLONE feature.
-   MCE::Shared::init($_wid) if (!$_use_threads && $INC{'MCE/Shared.pm'});
+   MCE::Shared::init() if (!$_use_threads && $INC{'MCE/Shared.pm'});
 
    _do_send_init($self);
 
    ## Call module's worker_init routine for modules plugged into MCE.
    for my $_p (@{ $_plugin_worker_init }) { $_p->($self); }
 
-   ## Begin processing if worker was added during processing. Otherwise,
-   ## respond back to the main process if the last worker spawned.
-   if (defined $_params) {
-      _worker_do($self, $_params);
-      undef $_params;
-   }
+   ## Begin processing if worker was added during processing.
+   _worker_do($self, $_params), undef $_params if defined($_params);
 
    ## Enter worker loop.
    _worker_loop($self);
@@ -727,3 +730,32 @@ sub _worker_main {
 
 1;
 
+__END__
+
+###############################################################################
+## ----------------------------------------------------------------------------
+## Module usage.
+##
+###############################################################################
+
+=head1 NAME
+
+MCE::Core::Worker - Core methods for the worker process
+
+=head1 VERSION
+
+This document describes MCE::Core::Worker version 1.866
+
+=head1 DESCRIPTION
+
+This package provides main, loop, and relevant methods used internally by
+the worker process.
+
+There is no public API.
+
+=head1 AUTHOR
+
+Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
+
+=cut
+
