@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.838';
+our $VERSION = '1.866';
 
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
@@ -86,7 +86,7 @@ sub import {
 use constant {
    MAX_DQ_DEPTH => 192,      # Maximum dequeue notifications
 
-   MUTEX_LOCKS  => 6,        # Number of mutex locks for 1st level defense
+   MUTEX_LOCKS  => 5,        # Number of mutex locks for 1st level defense
                              # against many workers waiting to dequeue
 
    OUTPUT_W_QUE => 'W~QUE',  # Await from the queue
@@ -114,8 +114,7 @@ use constant {
 ## _qr_sock _qw_sock _datp _datq _heap _id _init_pid _nb_flag _porder _type
 ## _ar_sock _aw_sock _asem _tsem
 
-my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
-my $_tid = $_has_threads ? threads->tid() : 0;
+my $_tid = $INC{'threads.pm'} ? threads->tid() : 0;
 
 my %_valid_fields_new = map { $_ => 1 } qw(
    await barrier fast gather porder queue type
@@ -125,12 +124,12 @@ my $_all = {};
 my $_qid = 0;
 
 sub CLONE {
-   $_tid = threads->tid() if $_has_threads;
+   $_tid = threads->tid() if $INC{'threads.pm'};
 }
 
 sub DESTROY {
    my ($_Q) = @_;
-   my $_pid = $_has_threads ? $$ .'.'. $_tid : $$;
+   my $_pid = $_tid ? $$ .'.'. $_tid : $$;
 
    delete $_all->{ $_Q->{_id} } if exists $_Q->{_id};
    undef $_Q->{_datp}, undef $_Q->{_datq}, undef $_Q->{_heap};
@@ -203,7 +202,7 @@ sub new {
 
    ## -------------------------------------------------------------------------
 
-   $_Q->{_init_pid} = $_has_threads ? $$ .'.'. $_tid : $$;
+   $_Q->{_init_pid} = $_tid ? $$ .'.'. $_tid : $$;
    $_Q->{_id} = ++$_qid; $_all->{$_qid} = $_Q;
    $_Q->{_dsem} = 0 if $_Q->{_fast};
 
@@ -221,7 +220,7 @@ sub new {
    MCE::Util::_sock_pair($_Q, qw(_ar_sock _aw_sock)) if $_Q->{_await};
 
    if (exists $_argv{queue} && scalar @{ $_argv{queue} }) {
-      MCE::Util::_syswrite($_Q->{_qw_sock}, $LF);
+      syswrite($_Q->{_qw_sock}, $LF);
    }
 
    return $_Q;
@@ -407,7 +406,7 @@ sub _heap_insert_high {
          $_Q->{_tsem} = $_t;
 
          if ($_Q->pending() <= $_t) {
-            MCE::Util::_syswrite($_Q->{_aw_sock}, $LF);
+            syswrite($_Q->{_aw_sock}, $LF);
          } else {
             $_Q->{_asem} += 1;
          }
@@ -460,7 +459,7 @@ sub _heap_insert_high {
                return;
             }
             if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-               MCE::Util::_syswrite($_Q->{_qw_sock}, $LF);
+               syswrite($_Q->{_qw_sock}, $LF);
             }
             push @{ $_Q->{_datq} }, @{ $_MCE->{thaw}($_buf) };
          }
@@ -504,7 +503,7 @@ sub _heap_insert_high {
                return;
             }
             if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-               MCE::Util::_syswrite($_Q->{_qw_sock}, $LF);
+               syswrite($_Q->{_qw_sock}, $LF);
             }
             push @{ $_Q->{_datq} }, $_;
          }
@@ -564,7 +563,7 @@ sub _heap_insert_high {
                if ($_pending) {
                   $_pending = MAX_DQ_DEPTH if ($_pending > MAX_DQ_DEPTH);
                   for my $_i (1 .. $_pending) {
-                     MCE::Util::_syswrite($_Q->{_qw_sock}, $LF);
+                     syswrite($_Q->{_qw_sock}, $LF);
                   }
                }
                $_Q->{_dsem} = $_pending;
@@ -575,11 +574,11 @@ sub _heap_insert_high {
          }
          else {
             ## Otherwise, never to exceed one byte in the channel
-            MCE::Util::_syswrite($_Q->{_qw_sock}, $LF) if $_Q->_has_data();
+            syswrite($_Q->{_qw_sock}, $LF) if $_Q->_has_data();
          }
 
          if (exists $_Q->{_ended} && !$_Q->_has_data()) {
-            MCE::Util::_syswrite($_Q->{_qw_sock}, $LF);
+            syswrite($_Q->{_qw_sock}, $LF);
          }
 
          if ($_cnt) {
@@ -587,7 +586,7 @@ sub _heap_insert_high {
             print {$_DAU_R_SOCK} length($_buf).'1'.$LF, $_buf;
          }
          elsif (defined $_buf) {
-            if (!looks_like_number $_buf && !ref $_buf) {
+            if (!ref $_buf) {
                print {$_DAU_R_SOCK} length($_buf).'0'.$LF, $_buf;
             } else {
                $_buf = $_MCE->{freeze}([ $_buf ]);
@@ -600,7 +599,7 @@ sub _heap_insert_high {
 
          if ($_Q->{_await} && $_Q->{_asem} && $_Q->pending() <= $_Q->{_tsem}) {
             for my $_i (1 .. $_Q->{_asem}) {
-               MCE::Util::_syswrite($_Q->{_aw_sock}, $LF);
+               syswrite($_Q->{_aw_sock}, $LF);
             }
             $_Q->{_asem} = 0;
          }
@@ -626,7 +625,7 @@ sub _heap_insert_high {
             my $_buf = $_Q->_dequeue();
 
             if (defined $_buf) {
-               if (!looks_like_number $_buf && !ref $_buf) {
+               if (!ref $_buf) {
                   print {$_DAU_R_SOCK} length($_buf).'0'.$LF, $_buf;
                } else {
                   $_buf = $_MCE->{freeze}([ $_buf ]);
@@ -660,7 +659,7 @@ sub _heap_insert_high {
 
          if ($_Q->{_await} && $_Q->{_asem} && $_Q->pending() <= $_Q->{_tsem}) {
             for my $_i (1 .. $_Q->{_asem}) {
-               MCE::Util::_syswrite($_Q->{_aw_sock}, $LF);
+               syswrite($_Q->{_aw_sock}, $LF);
             }
             $_Q->{_asem} = 0;
          }
@@ -732,7 +731,7 @@ sub _heap_insert_high {
             print {$_DAU_R_SOCK} '-1'.$LF;
          }
          else {
-            if (!looks_like_number $_buf && !ref $_buf) {
+            if (!ref $_buf) {
                print {$_DAU_R_SOCK} length($_buf).'0'.$LF, $_buf;
             } else {
                $_buf = $_MCE->{freeze}([ $_buf ]);
@@ -757,7 +756,7 @@ sub _heap_insert_high {
             print {$_DAU_R_SOCK} '-1'.$LF;
          }
          else {
-            if (!looks_like_number $_buf && !ref $_buf) {
+            if (!ref $_buf) {
                print {$_DAU_R_SOCK} length($_buf).'0'.$LF, $_buf;
             } else {
                $_buf = $_MCE->{freeze}([ $_buf ]);
@@ -858,7 +857,7 @@ sub _mce_m_end {
    my ($_Q) = @_;
 
    if (!exists $_Q->{_ended}) {
-      MCE::Util::_syswrite($_Q->{_qw_sock}, $LF) unless $_Q->{_nb_flag};
+      syswrite($_Q->{_qw_sock}, $LF) unless $_Q->{_nb_flag};
       $_Q->{_ended} = undef;
    }
 
@@ -877,7 +876,7 @@ sub _mce_m_enqueue {
       return;
    }
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      MCE::Util::_syswrite($_Q->{_qw_sock}, $LF);
+      syswrite($_Q->{_qw_sock}, $LF);
    }
 
    ## Append item(s) into the queue.
@@ -901,7 +900,7 @@ sub _mce_m_enqueuep {
       return;
    }
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      MCE::Util::_syswrite($_Q->{_qw_sock}, $LF);
+      syswrite($_Q->{_qw_sock}, $LF);
    }
 
    $_Q->_enqueuep($_p, @_);
@@ -945,7 +944,7 @@ sub _mce_m_dequeue {
          if ($_pending) {
             $_pending = MAX_DQ_DEPTH if ($_pending > MAX_DQ_DEPTH);
             for my $_i (1 .. $_pending) {
-               MCE::Util::_syswrite($_Q->{_qw_sock}, $LF);
+               syswrite($_Q->{_qw_sock}, $LF);
             }
          }
          $_Q->{_dsem} = $_pending;
@@ -956,11 +955,11 @@ sub _mce_m_dequeue {
    }
    else {
       ## Otherwise, never to exceed one byte in the channel
-      MCE::Util::_syswrite($_Q->{_qw_sock}, $LF) if $_Q->_has_data();
+      syswrite($_Q->{_qw_sock}, $LF) if $_Q->_has_data();
    }
 
    if (exists $_Q->{_ended} && !$_Q->_has_data()) {
-      MCE::Util::_syswrite($_Q->{_qw_sock}, $LF);
+      syswrite($_Q->{_qw_sock}, $LF);
    }
 
    $_Q->{_nb_flag} = 0;
@@ -1040,7 +1039,7 @@ sub _mce_m_insert {
       return;
    }
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      MCE::Util::_syswrite($_Q->{_qw_sock}, $LF);
+      syswrite($_Q->{_qw_sock}, $LF);
    }
 
    if (abs($_i) > scalar @{ $_Q->{_datq} }) {
@@ -1088,7 +1087,7 @@ sub _mce_m_insertp {
       return;
    }
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      MCE::Util::_syswrite($_Q->{_qw_sock}, $LF);
+      syswrite($_Q->{_qw_sock}, $LF);
    }
 
    if (exists $_Q->{_datp}->{$_p} && scalar @{ $_Q->{_datp}->{$_p} }) {
@@ -1269,13 +1268,13 @@ sub _mce_m_heap {
       if ($_lock_chn) {
          # inlined for performance
          $_dat_ex = sub {
-            my $_pid = $_has_threads ? $$ .'.'. $_tid : $$;
+            my $_pid = $_tid ? $$ .'.'. $_tid : $$;
             MCE::Util::_sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
                unless $_DAT_LOCK->{ $_pid };
          };
          $_dat_un = sub {
-            my $_pid = $_has_threads ? $$ .'.'. $_tid : $$;
-            MCE::Util::_syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
+            my $_pid = $_tid ? $$ .'.'. $_tid : $$;
+            syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
                if $_DAT_LOCK->{ $_pid };
          };
       }
@@ -1355,7 +1354,7 @@ sub _mce_m_heap {
          my $_buf = $_Q->{_id}.$LF . length($_tmp).$LF;
          $_req1->(OUTPUT_A_QUE, $_buf, $_tmp);
       }
-      elsif (!looks_like_number $_[0] && defined $_[0]) {
+      elsif (defined $_[0]) {
          my $_buf = $_Q->{_id}.$LF . length($_[0]).$LF;
          $_req1->(OUTPUT_S_QUE, $_buf, $_[0]);
       }
@@ -1381,7 +1380,7 @@ sub _mce_m_heap {
          my $_buf = $_Q->{_id}.$LF . $_p.$LF . length($_tmp).$LF;
          $_req1->(OUTPUT_A_QUP, $_buf, $_tmp);
       }
-      elsif (!looks_like_number $_[0] && defined $_[0]) {
+      elsif (defined $_[0]) {
          my $_buf = $_Q->{_id}.$LF . $_p.$LF . length($_[0]).$LF;
          $_req1->(OUTPUT_S_QUP, $_buf, $_[0]);
       }
@@ -1497,7 +1496,7 @@ sub _mce_m_heap {
 
       my ($_buf, $_tmp);
 
-      if (scalar @_ > 1 || looks_like_number $_[0] || ref $_[0] || !defined $_[0]) {
+      if (scalar @_ > 1 || ref $_[0] || !defined $_[0]) {
          $_tmp = $_MCE->{freeze}([ @_ ]);
          $_buf = $_Q->{_id}.$LF . $_i.$LF . (length($_tmp) + 1).$LF . $_tmp.'1';
       } else {
@@ -1523,7 +1522,7 @@ sub _mce_m_heap {
 
       my ($_buf, $_tmp);
 
-      if (scalar @_ > 1 || looks_like_number $_[0] || ref $_[0] || !defined $_[0]) {
+      if (scalar @_ > 1 || ref $_[0] || !defined $_[0]) {
          $_tmp = $_MCE->{freeze}([ @_ ]);
          $_buf = $_Q->{_id}.$LF . $_p.$LF . $_i.$LF .
             (length($_tmp) + 1).$LF . $_tmp.'1';
@@ -1602,7 +1601,7 @@ MCE::Queue - Hybrid (normal and priority) queues
 
 =head1 VERSION
 
-This document describes MCE::Queue version 1.838
+This document describes MCE::Queue version 1.866
 
 =head1 SYNOPSIS
 
@@ -1947,7 +1946,7 @@ by a call to dequeue.
 =head2 $q->insertp ( $p, $index, $item [, $item, ... ] )
 
 Adds the list of items to the queue at the specified index position with
-priority. The behavior is similarly to C<$q->insert> otherwise.
+priority. The behavior is similarly to C<< $q->insert >> otherwise.
 
 =head2 $q->pending ( void )
 
@@ -1985,7 +1984,7 @@ call to dequeue. Negative index values are supported, similarly to arrays.
 
 Returns an item from the queue with priority, at the specified index, without
 dequeuing anything. It defaults to the head of the queue if index is not
-specified. The behavior is similarly to C<$q->peek> otherwise.
+specified. The behavior is similarly to C<< $q->peek >> otherwise.
 
 =head2 $q->peekh ( [ $index ] )
 
@@ -2014,7 +2013,7 @@ numbers, not the data.
 
  @h = $q->heap;   # $MCE::Queue::HIGHEST
  # Heap contains: 6, 5, 4
- 
+
  @h = $q->heap;   # $MCE::Queue::LOWEST
  # Heap contains: 4, 5, 6
 
@@ -2022,21 +2021,21 @@ numbers, not the data.
 
 =over 3
 
-=item L<List::BinarySearch>
+=item * L<List::BinarySearch>
 
 The bsearch_num_pos method was helpful for accommodating the highest and lowest
 order in MCE::Queue.
 
-=item L<POE::Queue::Array>
+=item * L<POE::Queue::Array>
 
 For extra optimization, two if statements were adopted for checking if the item
 belongs at the end or head of the queue.
 
-=item L<List::Priority>
+=item * L<List::Priority>
 
 MCE::Queue supports both normal and priority queues.
 
-=item L<Thread::Queue>
+=item * L<Thread::Queue>
 
 Thread::Queue is used as a template for identifying and documenting the methods.
 
@@ -2051,7 +2050,7 @@ simultaneously; e.g.
 
  $q->pending();                  # counts both normal/priority queues
 
-=item L<Parallel::DataPipe>
+=item * L<Parallel::DataPipe>
 
 The recursion example, in the synopsis above, was largely adopted from this
 module.
